"""Constants for Indy."""

CATEGORY_CRED_DEF = "credential_def"
CATEGORY_CRED_DEF_PRIVATE = "credential_def_private"
CATEGORY_CRED_DEF_KEY_PROOF = "credential_def_key_proof"

CATEGORY_SCHEMA = "schema"

CATEGORY_REV_REG = "revocation_reg"
CATEGORY_REV_REG_DEF = "revocation_reg_def"
CATEGORY_REV_REG_DEF_PRIVATE = "revocation_reg_def_private"
CATEGORY_REV_REG_INFO = "revocation_reg_info"
CATEGORY_REV_REG_ISSUER = "revocation_reg_def_issuer"
